/**
 * @package   Crystalline Template - RocketTheme
 * @version   1.2 June 19, 2012
 * @author    RocketTheme http://www.rockettheme.com
 * @copyright Copyright (C) 2007 - 2012 RocketTheme, LLC
 * @license   http://www.rockettheme.com/legal/license.php RocketTheme Proprietary Use License
 */

var crIE9 = Browser.name == 'ie' && Browser.version == 9;
var PresetCreator = new Class({
	Implements: [Options, Events],
	initialize: function(options) {
		this.setOptions(options);

		this.popup = document.id('presets-popup');
		if (!this.popup) return;

		this.makePop();

		var self = this;

		document.id('preset-creator').addEvent('click', function(e) {
			e.stop();
			self.showPop();
		});

		Crystalline.newCSS = {};

		this.radios = $$('.popup-row input[type=radio]');

		document.id('presets').addEvent('change', this.presets.bind(this));
        this.aseInit();

		this.tips();
		this.colors();
		this.sliders();
		this.radioButtons();
		this.apply();
		this.initial = true;
	},
	presets: function() {
		var selected = document.id('popup-presets').getElement('.selected span').innerHTML, self = this;
		$H(Presets.get(selected)).each(function(value, key) {
			var color = {rgb: value.hexToRgb(true), hex: value};

			var mr = key.replace("-", "_");
			if (Crystalline.mooRainbow[mr]) {
				Crystalline.mooRainbow[mr].manualSet(color.rgb);
				Crystalline.mooRainbow[mr].fireEvent('onChange', color);
			} else if (key.contains('overlay') || key.contains('graphic')) {
				var type = key.contains('overlay') ? 'patterns' : 'headers';

				var patternKeys = Crystalline.sliders[type].getKeys();
				var patternValues = Crystalline[type].getKeys();
				var index = patternValues.indexOf(value);

				if (index < 0) index = 0;

				Crystalline.sliders[type].get(key).set(index).fireEvent('onComplete');
			} else if (key.contains('shadows')) {
				var name = key.replace("shadows", "").clean();
				document.id((name+value).clean()).fireEvent('click');
			}
		});
	},

	colors: function() {
		$$('.color-preview').each(function(color, i) {
			var cls = color.getParent().className.replace('popup-color ', '');
			var id = color.getParent().getParent().getParent().getParent().id;
			id = id.replace('popup-', '');


			var section = color.getRowID();
			var bit = color.getParent().className.replace('popup-color ', '');
			var item = section + "-" + bit;

			var moorainbow = new MooRainbow(color, {
				id: 'mr_'+id+'_' + cls,
				startColor: Crystalline.savedCSS.get(item).hexToRgb(true),
				imgPath: Crystalline.gantryURL+'/admin/widgets/colorchooser/images/',
				onChange: function(color) {
					this.element.setStyle('background-color', color.hex);

					Crystalline.newCSS[item] = color.hex;

				}
			});

			Crystalline.mooRainbow[item.replace("-", "_")] = moorainbow;
			color.setStyle('background-color', Crystalline.savedCSS.get(item));

			Crystalline.newCSS[item] = color.getStyle('background-color');
		});
	},

	sliders: function() {
		var sliders = $$('.popup-slider'), knobs = $$('.popup-knob'), self = this, dragging = false;
		var previews = $$('.overlay-preview');
		Crystalline.sliders['patterns'] = new Hash({});
		Crystalline.sliders['headers'] = new Hash({});
		sliders.each(function(slider, i) {
			var isHeader = slider.hasClass('overlays-header');
			var patterns = Crystalline[isHeader ? 'headers' : 'patterns'];
			var steps = patterns.getLength();
			var name = slider.getRowID() + "-" + (isHeader ? 'graphic' : 'overlay');

			var s = new RokSlider(slider, knobs[i], {
				steps: steps - 1,
				snap: true,
				onComplete: function() {
					var d = self.popup.retrieve('dragger');
					if (d) d.attach();
					dragging = false;
					this.knob.removeClass('down');

					var x = self.tips[0].getSize().x;
					self.tips.setStyle('left', (isHeader ? 90 : 120) - x / 2);

					var item = Crystalline[isHeader ? 'headers' : 'patterns'].getKeys()[this.step];
					var path = Crystalline[isHeader ? 'headers' : 'patterns'].get(item)['path'];

					Crystalline.newCSS[name] = item;

					new Asset.image(path);
				},
				onDrag: function(now) {
					dragging = true;
					this.element.getFirst().setStyle('width', now + 10);
					//self.tips.setStyle('visibility', 'visible');

					var currentKey = patterns.getKeys()[s.step];
					var pattern = patterns.get(currentKey);
					var name = pattern['name'];

					self.tips.inject(slider);
					self.tipSpan[0].innerHTML = name;
					var x = self.tips[0].getSize().x;
					self.tips.setStyle('left', (isHeader ? 90 : 120) - x / 2);

					var previous = slider.getPrevious(), preview;
					if (previous.hasClass('popup-overlaycolor')) {
						preview = previous.getElement('.overlay-preview');

						if (preview) preview.setStyle('background', 'url('+pattern['path']+') repeat');
					}
				},
				onTick: function(position) {
					var d = self.popup.retrieve('dragger');
					if (d) d.detach();
					if(this.options.snap) position = this.toPosition(this.step);
					this.knob.setStyle(this.property, position);
					this.fireEvent('onDrag', position);
				}
			});

			Crystalline.sliders[isHeader ? 'headers' : 'patterns'].set(name, s);

			var saved = Crystalline.savedCSS.get(name);
			var keys = Crystalline[isHeader ? 'headers' : 'patterns'].getKeys();
			var index = keys.indexOf(saved);

			s.set(index);

			dragging = false;
			self.tips.setStyle('visibility', 'hidden');

			slider.addEvents({
				'mouseenter': function() {
					self.tips.inject(slider);
					var currentKey = patterns.getKeys()[s.step];
					var pattern = patterns.get(currentKey);
					var name = pattern['name'];

					self.tipSpan[0].innerHTML = name;
					var x = self.tips[0].getSize().x;

					if (!isHeader) {
						self.tips.setStyles({
							'visibility': 'visible',
							'top': -90,
							'left': 120 - x / 2
						});
					} else {
						self.tips.setStyles({
							'visibility': 'visible',
							'top': -60,
							'left': 90 - x / 2
						});
					}
				},
				'mouseleave': function() {
					self.tips.setStyle('visibility', 'hidden');
				}
			});
			knobs[i].addEvents({
				'mousedown': function() {this.addClass('down');},
				'mouseup': function() {this.removeClass('down');}
			});
		});

		var d = self.popup.retrieve('dragger');
		if (d) d.attach();
	},

	tips: function() {
		this.tips = $$('.popup-tip').setStyle('visibility', 'hidden');
		this.tipSpan = $$('.popup-tip').getElement('span');
	},

	radioButtons: function() {
		this.radios.each(function(radio, i) {
			var row = radio.getRowID();
			var name = row + "-" + 'shadows';
			var item = document.id(row + '-' + Crystalline.savedCSS.get(name));
			radio.addEvent('click', function() {
				Crystalline.newCSS[name] = this.value;
			});
			if (i % 3 == 2) {
				if (item) item.fireEvent('click');
				if (Browser.Engine.presto && item) item.fireEvent('click');
			}
		});
	},

	makePop: function() {
		this.popup.setStyles({
			'top': -99999,
			'opacity': 1,
			'visibility': 'visible'
		}).inject(document.body);

		if (!Browser.Engine.presto) this.popup.makeDraggable();

		this.popupFx = new Fx.Tween(this.popup, {
			transition: Fx.Transitions.Back.easeOut, duration: 600,
			onStart: function(){
				if (!this.popup.getStyle('opacity').toInt()) this.popup.setStyle('visibility', 'visible');
			}.bind(this)
		}).set('opacity', 0);
	},

	reposPop: function() {
		var win = window.getSize();
		var width = this.popup.getStyle('width').toInt(), height = this.popup.getStyle('height').toInt();

	},

	showPop: function() {
		if (this.initial) {
			this.initial = false;
		}
		var win = window.getSize();
		var width = this.popup.getStyle('width').toInt(), height = this.popup.getStyle('height').toInt();

		this.popup.setStyle('display', 'block');
		this.popup.setStyles({
			'top': ((win.y / 2) + window.getScroll().y) - (height / 2),
			'left': (win.x / 2) - (width / 2)
		});
		if (Browser.Engine.trident && !crIE9) this.popupFx.set('opacity', 1);
		else this.popupFx.start('opacity', 1);
	},

	build: function() {
		var ids = ['presets', 'header', 'main', 'feature', 'body', 'bottom', 'palette'];
		(7).times(function(i) {
			var row = new Element('div', {'class': 'popup-row', id: 'popup-' + ids[i]}).inject(this.popup);
			var title = new Element('div', {'class': 'popup-title'}).inject(row).adopt(
				new Element('span').set('text', ids[i])
			);
			var content = new Element('div', {'class': 'popup-content'}).inject(row);
		}.bind(this));
	},

	apply: function() {
		var apply = document.id('popup-apply'), close = document.id('popup-close'), self = this;
		var morph = new Fx.Morph(document.id('presets-popup'), {duration: 300});

		document.id('popup-minimize').addEvent('click', function(e) {
			e.stop();

			$H(Crystalline.mooRainbow).each(function(mr) {
				mr.hide();
			});

			if (this.hasClass('big')) {
				this.removeClass('big').addClass('small');
				$$('.popup-row').setStyle('display', 'none');
				morph.start({
				    'width': 170,
				    'height': 50
				}).chain(function() {
				    document.id('presets-popup').setStyle('background-image', 'url('+Crystalline.templateUrl+'/images/popup/popup-bg-small.png)');

					document.id('popup-apply').inject(document.id('popup-miniature').setStyle('display', 'block'));
				});
			}
			else {
				this.removeClass('small').addClass('big');
				document.id('presets-popup').setStyle('background-image', 'url('+Crystalline.templateUrl+'/images/popup/popup-bg.png)');

				document.id('popup-miniature').setStyle('display', 'none');
				morph.start({
				   'width': 534,
				   'height': 560
				}).chain(function() {
					document.id('popup-apply').inject($$('#popup-presets .controls')[0]);
				   	$$('.popup-row').setStyle('display', 'block');
				});
			}
		});

		if (apply) {
			apply.addEvent('click', function(e) {
				e.stop();

				var newSettings = new Hash({}), effects = {};

				Crystalline.savedCSS.each(function(value, key) {
					var fixedVal = value.toLowerCase();
					if (fixedVal.length == 4 && fixedVal.substr(0, 1) == "#") {
						var v = fixedVal;
						fixedVal = "#" + v.substr(1, 1) + v.substr(1, 1) + v.substr(2, 1) + v.substr(2, 1) + v.substr(3, 1) + v.substr(3, 1);
					}

					if (Crystalline.newCSS[key] && Crystalline.newCSS[key] != fixedVal) {
						Crystalline.savedCSS.set(key, Crystalline.newCSS[key]);
						newSettings.set(key, Crystalline.newCSS[key]);
					}
				});

				var variationsException = new Hash({});
				newSettings.each(function(value, key) {
					Cookie.write(Crystalline.cookiePrefix+'-' + key, value, {duration: Crystalline.cookieTime, path: Crystalline.baseUrl});

					if (key.contains('background') || key.contains('text') || key.contains('link')) {
						var isBg = key.contains('background');

						var regular = Crystalline.cssRules.get(key).split(", ");
						var variations = Crystalline.varRules.get(key).split(", ");

						regular = regular.slice(0, - variations.length);

						regular =  (regular.length) ? regular.join(", ") : null;
						variations = (variations.length) ? variations.join(", ") : null;

						variationsException.set(key, [variations, value]);

						$$(regular).each(function(rule, i) {
							new Fx.Tween(rule, {duration: 300, link: 'cancel'}).start((isBg ? 'backgroundColor' : 'color'), value);
						});
						$$(variations).each(function(rule, i) {
							new Fx.Tween(rule, {duration: 300, link: 'cancel'}).start((isBg ? 'backgroundColor' : 'color'), value);
						});
					}
				});

				var variations = Hash.combine(Crystalline.savedCSS, Crystalline.newCSS);

				variations.each(function(value, key) {

					if (key.contains('background') || key.contains('text') || key.contains('link')) {
						var isBg = key.contains('background');

						$$(Crystalline.varRules.get(key)).each(function(rule, i) {
							new Fx.Tween(rule, {duration: 300, link: 'cancel'}).start((isBg ? 'backgroundColor' : 'color'), value);
						});
					}
				});

				$H(Crystalline.sliders).each(function(value, key) {
					value.each(function(value2, key2) {
						var storedValue = Crystalline[key].getKeys()[value2.step];
						//Cookie.set(Crystalline.cookiePrefix+'-' + key2, storedValue, {duration: Crystalline.cookieTime});

						var type = key2.contains('overlay') ? 'overlay' : 'graphic';
						var cleanName = key2.replace('overlay', '').replace('graphic', '');
						var id = 'rt-' + cleanName + type;

						if (cleanName == 'bottom-') {
							var footer = document.id('rt-footer-overlay');
							if (footer) {
								footer.className = "";
								footer.addClass(key2 + '-' + storedValue);
							}
						}

						var el = document.id(id);
						if (el) {
							el.className = "";
							el.addClass(key2 + '-' + storedValue);
						}
					});
				});

				var radiosValues = self.radios.filter(function(radio) {
				    return radio.checked;
				}).get('value');

				var i = 0;
				Crystalline.shadows.each(function(value, key) {
					var items = '#' + value.join(',#');
					items = $$(items.split(','));
					items.each(function(item) {
						if (key == 'header' || key == 'header-') {
							var menutop = $$('.menutop');
							menutop.each(function(menu) {
								menu.removeClass('header-shadows-none');
								menu.removeClass('header-shadows-light');
								menu.removeClass('header-shadows-dark');
								menu.removeClass('menutop').addClass('header-shadows-' + radiosValues[i]).addClass('menutop');
							});
						}

						item.className = key + "shadows-" + radiosValues[i];
						//Cookie.set(Crystalline.cookiePrefix+'-' + key + "Shadows", radiosValues[i], {duration: Crystalline.cookieTime});
					});
					i++;
				});
			});
		};

		if (close) {
			close.addEvent('click', function(e) {
				e.stop();
				$H(Crystalline.mooRainbow).each(function(mr) {
					mr.hide();
				});
				document.id('presets-popup').set('tween', {duration: 300, link: 'chain', onComplete: function(){
					document.id('presets-popup').setStyle('visibility', 'hidden');
				}});
				document.id('presets-popup')[(Browser.Engine.trident && !crIE9) ? 'setStyle' : 'tween']('opacity', 0);
				(function() {
				    if (document.id('popup-minimize').hasClass('small')) {
						document.id('popup-minimize').removeClass('small').addClass('big');
						document.id('presets-popup').setStyle('background-image', 'url('+Crystalline.templateUrl+'/images/popup/popup-bg.png)');

						document.id('popup-miniature').setStyle('display', 'none');
						document.id('presets-popup').setStyles({
						   'width': 534,
						   'height': 560
						});
						document.id('popup-apply').inject($$('#popup-presets .controls')[0]);
						$$('.popup-row').setStyle('display', 'block');
					}

					self.reposPop();
				}).delay(300);
			});
		}
	},

	makeDraggables: function() {
		if (this.aseColors.length) {

			var droppables = $$('.popup-colors .popup-color'), self = this;
			var droppablesCopy = $A(droppables);
			var rainbowDroppables = $$(Array.combine(droppables.clone(), this.aseColors.clone()));

			droppables.each(function(drag) {
				var mrDroppables = $A(Array.combine(droppables, this.aseColors));
				mrDroppables.erase(drag);
				mrDroppables = $$(mrDroppables);
				new Drag.Move(drag, {
					onStart: function(element, event) {
						var color = this.backupElement.getElement('.color-preview').getStyle('background-color');

						var sizes = {
							x: this.mouse.start.x - this.backupElement.getPosition().x - 10,
							y: this.mouse.start.y - this.backupElement.getPosition().y - 10
						};

						this.moveable.setStyles({
							'background-color': color,
							'border': '1px solid #000',
							'display': 'none',
							'top': sizes.y,
							'left': sizes.x
						});
					},
					onBeforeStart: function(element, event) {
						var d = self.popup.retrieve('dragger');
						if (d) d.detach();
						if (!this.backupElement) this.backupElement = this.element;

						var sizes = {
							x: this.mouse.start.x - this.backupElement.getPosition().x - 10,
							y: this.mouse.start.y - this.backupElement.getPosition().y - 10
						};

						if (!this.moveable) {
							this.moveable = new Element('div', {
								'styles': {
									'width': 20,
									'height': 20,
									'display': 'none',
									'background-color': '#fff',
									'position': 'absolute',
									'top': sizes.y,
									'left': sizes.x,
									'z-index': 58050
								}
							}).inject(this.element);

							if (Browser.Engine.webkit || Browser.Engine.gecko) {
								var prefix = (Browser.Engine.webkit) ? '-webkit-' : '-moz-';

								this.moveable.setStyle(prefix + 'border-radius', 3);
							}
						}

						this.moveable.setStyles({'top': sizes.y, 'left': sizes.x});
						this.element = this.moveable;

					},
					onDrag: function() {
						this.moveable.setStyle('display', 'block');
					},
					onComplete: function() {
						var d = self.popup.retrieve('dragger');
						if (d) d.attach();
						this.moveable.setStyle('display', 'none');
						this.element = this.backupElement;
					},
					droppables: mrDroppables,
					onLeave: function(dragger, droppable) {
						droppable = document.id(droppable);
						if (droppable && droppable.getElement('span')) droppable.getElement('span').setStyle('color', '#bbb');
					},
					onEnter: function(dragger, droppable) {
						droppable = document.id(droppable);
						if (droppable && droppable.getElement('span')) droppable.getElement('span').setStyle('color', '#f0d124');
					},
					onDrop: function(dragger, droppable) {
						if (!droppable) return;
						var color = dragger.getStyle('background-color');
						var row = droppable.getRowID();
						if (row == 'palette') {
							droppable.setStyle('background-color', color);
							dragger.setStyle('display', 'none');
						} else {
							var mr = row + "_" + droppable.className.replace('popup-color ', '').replace("-", "_");
							var newColor = {hex: color, rgb: color.hexToRgb(true)};

							Crystalline.mooRainbow[mr].manualSet(newColor.rgb);
							Crystalline.mooRainbow[mr].fireEvent('onChange', newColor);
						}

						this.fireEvent('leave', [dragger, droppable]);
					}
				});

			}, this);
			this.aseColors.each(function(aseColor, i) {
				aseColor.store('cryst:aseX', aseColor.getPosition().x + (document.dir == 'ltr') ? (23*i) : -145);
			});
			this.aseColors.each(function(aseColor, i) {
				aseColor.setStyles({'top': 0, 'left': aseColor.retrieve('cryst:aseX', 0), 'position': 'absolute'});
				new Drag.Move(aseColor, {
					onBeforeStart: function() {
						var d = self.popup.retrieve('dragger');
						if (d) d.detach();
					},
					onComplete: function() {
						var d = self.popup.retrieve('dragger');
						if (d) d.attach();
					},
					'droppables': droppablesCopy,
					onLeave: function(dragger, droppable) {
						droppable = document.id(droppable);
						if (droppable) droppable.getElement('span').setStyle('color', '#bbb');
					},
					onEnter: function(dragger, droppable) {
						droppable = document.id(droppable);
						if (droppable) droppable.getElement('span').setStyle('color', '#FF6015');
					},
					onDrop: function(dragger, droppable) {
						droppable = document.id(droppable);
						if (!droppable) return;
						var mr = droppable.getRowID() + "_" + droppable.className.replace('popup-color ', '').replace("-", "_");
						var color = dragger.getStyle('background-color');
						var newColor = {hex: color, rgb: color.hexToRgb(true)};

						Crystalline.mooRainbow[mr].manualSet(newColor.rgb);
						Crystalline.mooRainbow[mr].fireEvent('onChange', newColor);

						dragger.set('tween', {duration: 100});
						dragger.tween('opacity', 0);
						(function() {
							dragger.setStyles({'top': 0, 'left': aseColor.retrieve('cryst:aseX', 0)});
							dragger.tween('opacity', 1);
						}).delay(100);

						this.fireEvent('leave', [dragger, droppable]);
					}
				});

				aseColor.addEvent('emptydrop', function(obj) {
					this.set('morph', {duration: 300});
					this.morph({
						'top': 0,
						'left': this.retrieve('cryst:aseX', 0)
					});
				});
			});
		}

		$$('.popup-color').setStyles({'left': 0, 'top': 0});
	},

	aseInit: function() {
		var aseLoadTarget = document.id('ase-load-target'), aseSaveTarget = document.id('ase-save-target');
		var aseLoadForm = document.id('ase-load-form'), aseFile = document.id('ase-file');
		var aseSaveForm = document.id('ase-save-form'), aseSave = document.id('ase-save');

		var self = this;
		if (!aseLoadForm) return;

		if (Browser.Engine.trident && !crIE9) {aseFile.setStyle('opacity', 0.0001);}
		this.aseColors = $$('.ase-color');

		if (Browser.Engine.webkit || Browser.Engine.gecko) {
			var prefix = (Browser.Engine.webkit) ? '-webkit-' : '-moz-';

			this.aseColors.setStyle(prefix + 'border-radius', 3);
		}

		if (Browser.Engine.webkit) aseFile.setStyles({'right': 'auto', 'left': 0, 'cursor': 'default'});

		this.makeDraggables();

		aseFile.addEvent('change', function() {
			var symbol = (Crystalline.aseLoadUrl.contains('?')) ? '&' : '?';
			aseLoadForm.setProperty('action', Crystalline.aseLoadUrl + symbol + "cache=" + $time());
			aseLoadForm.setProperty('target', 'ase-load-target');

			aseLoadForm.submit();
		});

		aseSave.addEvent('click', function(e) {
			e = e.stop();

			aseSaveForm.setProperty('action', Crystalline.aseSaveUrl + "?cache=" + $time());
			aseSaveForm.setProperty('target', 'ase-save-target');

			var colors = self.aseColors.getStyle('background-color').map(function(color) {
				return new Color(color).hex;
			});

			document.id('ase-colors-list').value = colors.join(",");

			aseSaveForm.submit();
		});

		aseLoadForm.addEvent('submit', function(e) {
			e = e.stop();

			self.aseLoadSubmit();
		});

		aseLoadTarget.addEvent('load', function() {
			aseSaveForm.setProperty('action', 'index.php');
			var doc = this.contentDocument || this.contentWindow;
			if (doc.location == 'about:blank') return;

			var body = (Browser.Engine.trident && !crIE9) ? this.contentWindow.document.body : doc.body;

			self.aseLoadLoad(document.id(body));
		});

		aseSaveForm.addEvent('submit', function(e) {
			e = e.stop();

			self.aseSaveSubmit();
		});

		aseSaveTarget.addEvent('load', function() {
			aseSaveForm.setProperty('action', 'index.php');
			var doc = this.contentDocument || this.contentWindow;
			if (doc.location == 'about:blank') return;

			var body = (Browser.Engine.trident && !crIE9) ? this.contentWindow.document.body : doc.body;

			self.aseSaveLoad(document.id(doc[Browser.Engine.trident5 ? 'document' : 'body']));
		});
	},

	aseLoadSubmit: function(aseTarget) {
	},

	aseLoadLoad: function(response) {
		var content = response.innerHTML.trim().clean();
		var json = JSON.decode(content);
		var hash = new Hash(json);


		if (json && json.colors && hash.get('colors').length) {
			var colors = hash.get('colors');
			this.aseColors.setStyle('background-color', '#fff');
			colors.each(function(color, i) {
				if (i > 9) return;

				this.aseColors[i].setStyle('background-color', new Color(color).hex);
			}, this);
		} else {
			if (typeof console != 'undefined') console.error("Error while loading the ASE File. Response: ", response);
		}
	},

	aseSaveSubmit: function(aseTarget) {
	},

	aseSaveLoad: function(response) {
	}
});

MooRainbow.prototype.rePosition = function() {
	var coords = this.element.getCoordinates();
	this.layout.setStyles({
		'left': coords.left + 70,
		'top': coords.top - 20
	});
};

SelectBox.prototype.select = function(objs, index) {
	if (index == -1) return;
	objs.selected.set('html', objs.list[index].innerHTML);
	objs.real.selectedIndex = index;

	objs.real.fireEvent('change', index);
};
Element.implement({
	getRowID: function(el) {
		var matched = [];
		var cur = this.getParent();
		while (cur && cur !== (document.id('presets-popup') || document)) {
			if(cur.hasClass('popup-row')) matched.push(cur.id);
			cur = cur.getParent();
		}
		return matched[0].replace('popup-', '');
	}
});

window.addEvent('domready', function() {new PresetCreator();});
