<?php
/**
 * @package		Gantry Template Framework - RocketTheme
 * @version		1.2 June 19, 2012
 * @author		RocketTheme http://www.rockettheme.com
 * @copyright 	Copyright (C) 2007 - 2012 RocketTheme, LLC
 * @license		http://www.gnu.org/licenses/gpl-2.0.html GNU/GPLv2 only
 *
 * Gantry uses the Joomla Framework (http://www.joomla.org), a GNU/GPLv2 content management system
 *
 */
defined('JPATH_BASE') or die();

gantry_import('core.gantryfeature');

class GantryFeaturePresetCreator extends GantryFeature {
    var $_feature_name = 'presetcreator';

    function isEnabled() {
        global $gantry;
		
		if (!$gantry->isAdmin() && $gantry->browser->name == 'ie' && $gantry->browser->shortversion == '6') {
			return false;
		} else {
			return $gantry->get('colorchooser-enabled');
    	}
	}

	function isInPosition($position) {
		if ($position == "presetpopup") return true;
		else return false;
	}
	
	function isOrderable(){
		return false;
	}
	
	function init() {
        global $gantry;

		gantry_import('core.gantryjson');
		
		$gantry->addStyle('preset-creator.css');
		$gantry->addScript($gantry->gantryUrl . "/admin/widgets/colorchooser/js/mooRainbow.js");
		$gantry->addScript($gantry->gantryUrl . "/admin/widgets/slider/js/slider.js");
		$gantry->addScript($gantry->gantryUrl . "/admin/widgets/selectbox/js/selectbox.js");
		
		if ($this->_isAdmin()) {
			$gantry->addInlineScript("
				var GantryLang = {
					'preset_title': '" . JText::_('PRESET_TITLE') . "',
					'preset_select': '" . JText::_('PRESET_SELECT') . "',
					'preset_name': '" . JText::_('PRESET_NAME') . "',
					'key_name': '" . JText::_('KEY_NAME') . "',
					'preset_naming': '" . JText::_('PRESET_NAMING') . "',
					'preset_skip': '" . JText::_('PRESET_SKIP') . "',
					'success_save': '" . JText::_('SUCCESS_SAVE') . "',
					'success_msg': '" .JText::_('SUCCESS_MSG') . "',
					'fail_save': '" . JText::_('FAIL_SAVE') . "',
					'fail_msg': '" . JText::_('FAIL_MSG') . "',
					'cancel': '" . JText::_('CANCEL') . "',
					'save': '" . JText::_('SAVE') . "',
					'retry': '" . JText::_('RETRY') . "',
					'close': '" . JText::_('CLOSE') . "',
					'show_parameters': '" . JText::_('SHOW_PARAMETERS') . "'
				};
			");
			$gantry->addScript('preset-saver.js');
		}
		$gantry->addScript('preset-creator.js');
		
		$gantry->addInlineScript($this->_loadPresets());
		
		$cookie_time = $gantry->get('cookie_time') / 86400000 * 1000;
		
		$this->_jsInitialization($cookie_time);
		
	}
	
	function _isAdmin() {
		$user =& JFactory::getUser();
		if ($user->usertype == "Super Administrator") return true;
		else return false;		
	}
	
	function _loadPresets() {
		global $gantry;
		
		$output = "";
		
		foreach($gantry->presets['presets'] as $key => $preset) {
            $preset_name = $preset['name'];
			$output .= "'$preset_name': {";
			foreach($preset as $keyName => $preset_value) {
                if ($keyName != 'name'){
				    $output .= "'$keyName': '$preset_value', ";
                }
			}
			$output = substr($output, 0, -2);
			$output .= "}, ";
		}

		$output = substr($output, 0, -2);
		
		$output = 'Presets = new Hash({'.$output.'});';
		
		return "var Presets = {}; " . $output;
	}
	
	function render($position = "presetpopup") {
		global $gantry;
		
		$shadows = array(
			'header' => $gantry->get('header-shadows'),
			'main' => $gantry->get('main-shadows'),
			'feature' => $gantry->get('feature-shadows'),
			'body' => $gantry->get('body-shadows'),
			'bottom' => $gantry->get('bottom-shadows')
		);
		
		ob_start();
		?>
		<div id="presets-popup">
			<div class="popup-toolbar">
				<a href="#" id="popup-minimize" class="big"><span>min</span></a>
				<a href="#" id="popup-close"><span>close</span></a>
			</div>
			<div id="popup-miniature"></div>
			<div class="popup-row" id="popup-presets">
				<div class="popup-title"><span>Presets</span></div>
				<div class="popup-content">
					<div class="selectbox-container">
						<span class="popup-spantitle">Load Preset</span>
						<div class="selectbox-wrapper">	
							<div class="selectbox">		
								<?php
									$lis = $dropdownSpan = "<li>Choose Preset</li>";
									
									foreach($gantry->presets['presets'] as $preset_key => $preset_values) {
										$lis .=  "<li>".$preset_values['name']."</li>";
										$dropdownSpan = $preset_values['name'];
									}

								?>
								<div class="selectbox-top">			
									<div class="selected"><span>Choose Preset</span></div>
									<div class="arrow"></div>
								</div>
								<div class="selectbox-dropdown">
									<ul>
										<?php echo $lis; ?>
									</ul>
								<div class="selectbox-dropdown-bottom">
									<div class="selectbox-dropdown-bottomright"></div>
								</div>
							</div>
						</div>
						<select id="presets" name="params[presets]" class="selectbox-real">
								<option value="_disabled_" class="disabled" selected="selected">Choose Preset</option>
							<?php
								foreach($gantry->presets['presets'] as $preset_key => $preset_values) {
									echo "<option value='".$preset_key."'>".$preset_values['name']."</option>";
								}
							?>
						</select>
					</div>
					<div class="clr"></div>
					<div class="controls">
						<?php if ($this->_isAdmin()): ?>
							<button id="popup-savenew" class="preset-saver"><span>Save New</span></button>
						<?php endif; ?>
						<button id="popup-apply"><span>Apply</span></button>
					</div>
					</div>
				</div>
			</div>
			<div class="popup-row" id="popup-header">
				<div class="popup-title"><span>Header</span></div>
				<div class="popup-content">
					<div class="popup-colors">
						<div class="popup-color background">
							<span>Background</span>
							<div class="color-preview"></div>
						</div>
						<div class="popup-color text">
							<span>Text</span>
							<div class="color-preview"></div>
						</div>
						<div class="popup-color link">
							<span>Link</span>
							<div class="color-preview"></div>
						</div>
					</div>
					<div class="popup-overlay">
						<div class="popup-overlaycolor">
							<span>Overlay</span>
							<div class="overlay-preview"><div></div></div>
						</div>
						<div class="popup-slider">
							<div class="popup-sliderinner"></div>
							<div class="popup-knob"></div>
						</div>

						<div class="popup-radios">
							<span>Shadows</span>
							<div class="popup-shadows">
								<input type="radio" name="header-radio" id="header-none" value="none" alt="None" <?php echo ($shadows['header'] == 'none') ? 'checked="checked"' : ''; ?> />
								<label for="header-none">None</label>
								<input type="radio" name="header-radio" id="header-light" value="light" alt="Light" <?php echo ($shadows['header'] == 'light') ? 'checked="checked"' : ''; ?> />
								<label for="header-light">Light</label>
								<input type="radio" name="header-radio" id="header-dark" value="dark" alt="Dark" <?php echo ($shadows['header'] == 'dark') ? 'checked="checked"' : ''; ?> />
								<label for="header-dark">Dark</label>
							</div>
						</div>
					</div>
					<div class="popup-headeroverlay" style="height: 24px;">
						<span>Header Graphic</span>
						<div class="popup-slider overlays-header">
							<div class="popup-sliderinner"></div>
							<div class="popup-knob"></div>
						</div>
					</div>
				</div>
			</div>
			<div class="popup-row" id="popup-main">
				<div class="popup-title"><span>Main</span></div>
				<div class="popup-content">
					<div class="popup-colors">
						<div class="popup-color background">
							<span>Background</span>
							<div class="color-preview"></div>
						</div>
						<div class="popup-color text">
							<span>Text</span>
							<div class="color-preview"></div>
						</div>
						<div class="popup-color link">
							<span>Link</span>
							<div class="color-preview"></div>
						</div>
					</div>
					<div class="popup-overlay">
						<div class="popup-overlaycolor">
							<span>Overlay</span>
							<div class="overlay-preview"><div></div></div>
						</div>
						<div class="popup-slider">
							<div class="popup-sliderinner"></div>
							<div class="popup-knob"></div>
						</div>

						<div class="popup-radios">
							<span>Shadows</span>
							<div class="popup-shadows">
								<input type="radio" name="main-radio" id="main-none" value="none" alt="None"  <?php echo ($shadows['main'] == 'none') ? 'checked="checked"' : ''; ?> />
								<label for="main-none">None</label>
								<input type="radio" name="main-radio" id="main-light" value="light" alt="Light" <?php echo ($shadows['main'] == 'light') ? 'checked="checked"' : ''; ?> />
								<label for="main-light">Light</label>
								<input type="radio" name="main-radio" id="main-dark" value="dark" alt="Dark" <?php echo ($shadows['main'] == 'dark') ? 'checked="checked"' : ''; ?> />
								<label for="main-dark">Dark</label>
							</div>
						</div>
					</div>
				</div>
			</div>
			<div class="popup-row" id="popup-feature">
				<div class="popup-title"><span>Feature</span></div>
				<div class="popup-content">
					<div class="popup-colors">
						<div class="popup-color background">
							<span>Background</span>
							<div class="color-preview"></div>
						</div>
						<div class="popup-color text">
							<span>Text</span>
							<div class="color-preview"></div>
						</div>
						<div class="popup-color link">
							<span>Link</span>
							<div class="color-preview"></div>
						</div>
					</div>
					<div class="popup-overlay">
						<div class="popup-overlaycolor">
							<span>Overlay</span>
							<div class="overlay-preview"><div></div></div>
						</div>
						<div class="popup-slider">
							<div class="popup-sliderinner"></div>
							<div class="popup-knob"></div>
						</div>

						<div class="popup-radios">
							<span>Shadows</span>
							<div class="popup-shadows">
								<input type="radio" name="feature-radio" id="feature-none" value="none" alt="None" <?php echo ($shadows['feature'] == 'none') ? 'checked="checked"' : ''; ?> />
								<label for="feature-none">None</label>
								<input type="radio" name="feature-radio" id="feature-light" value="light" alt="Light" <?php echo ($shadows['feature'] == 'light') ? 'checked="checked"' : ''; ?> />
								<label for="feature-light">Light</label>
								<input type="radio" name="feature-radio" id="feature-dark" value="dark" alt="Dark" <?php echo ($shadows['feature'] == 'dark') ? 'checked="checked"' : ''; ?> />
								<label for="feature-dark">Dark</label>
							</div>
						</div>
					</div>
				</div>
			</div>
			<div class="popup-row" id="popup-body">
				<div class="popup-title"><span>Body</span></div>
				<div class="popup-content">
					<div class="popup-colors">
						<div class="popup-color background">
							<span>Background</span>
							<div class="color-preview"></div>
						</div>
						<div class="popup-color text">
							<span>Text</span>
							<div class="color-preview"></div>
						</div>
						<div class="popup-color link">
							<span>Link</span>
							<div class="color-preview"></div>
						</div>
					</div>
					<div class="popup-overlay">
						<div class="popup-overlaycolor">
							<span>Overlay</span>
							<div class="overlay-preview"><div></div></div>
						</div>
						<div class="popup-slider">
							<div class="popup-sliderinner"></div>
							<div class="popup-knob"></div>
						</div>

						<div class="popup-radios">
							<span>Shadows</span>
							<div class="popup-shadows">
								<input type="radio" name="body-radio" id="body-none" value="none" alt="None" <?php echo ($shadows['body'] == 'none') ? 'checked="checked"' : ''; ?> />
								<label for="body-none">None</label>
								<input type="radio" name="body-radio" id="body-light" value="light" alt="Light" <?php echo ($shadows['body'] == 'light') ? 'checked="checked"' : ''; ?> />
								<label for="body-light">Light</label>
								<input type="radio" name="body-radio" id="body-dark" value="dark" alt="Dark" <?php echo ($shadows['body'] == 'dark') ? 'checked="checked"' : ''; ?> />
								<label for="body-dark">Dark</label>
							</div>
						</div>
					</div>
				</div>
			</div>
			<div class="popup-row" id="popup-bottom">
				<div class="popup-title"><span>Bottom</span></div>
				<div class="popup-content">
					<div class="popup-colors">
						<div class="popup-color background">
							<span>Background</span>
							<div class="color-preview"></div>
						</div>
						<div class="popup-color text">
							<span>Text</span>
							<div class="color-preview"></div>
						</div>
						<div class="popup-color link">
							<span>Link</span>
							<div class="color-preview"></div>
						</div>
					</div>
					<div class="popup-overlay">
						<div class="popup-overlaycolor">
							<span>Overlay</span>
							<div class="overlay-preview"><div></div></div>
						</div>
						<div class="popup-slider">
							<div class="popup-sliderinner"></div>
							<div class="popup-knob"></div>
						</div>

						<div class="popup-radios">
							<span>Shadows</span>
							<div class="popup-shadows">
								<input type="radio" name="bottom-radio" id="bottom-none" value="none" alt="None" <?php echo ($shadows['bottom'] == 'none') ? 'checked="checked"' : ''; ?> />
								<label for="bottom-none">None</label>
								<input type="radio" name="bottom-radio" id="bottom-light" value="light" alt="Light" <?php echo ($shadows['bottom'] == 'light') ? 'checked="checked"' : ''; ?> />
								<label for="bottom-light">Light</label>
								<input type="radio" name="bottom-radio" id="bottom-dark" value="dark" alt="Dark" <?php echo ($shadows['bottom'] == 'dark') ? 'checked="checked"' : ''; ?> />
								<label for="bottom-dark">Dark</label>
							</div>
						</div>
					</div>
				</div>
			</div>
			<div class="popup-row" id="popup-palette">
				<div class="popup-title"><span>Palette</span></div>
				<div class="popup-content">
						<div class="ase-colors-wrapper">
						<div id="ase-color1" class="ase-color"></div>
						<div id="ase-color2" class="ase-color"></div>
						<div id="ase-color3" class="ase-color"></div>
						<div id="ase-color4" class="ase-color"></div>
						<div id="ase-color5" class="ase-color"></div>
						<div id="ase-color6" class="ase-color"></div>
						<div id="ase-color7" class="ase-color"></div>
						<div id="ase-color8" class="ase-color"></div>
						<div id="ase-color9" class="ase-color"></div>
						<div id="ase-color10" class="ase-color"></div>
						</div>
						<div class="ase-controls">
		                    <form id="ase-load-form" action="index.php" enctype="multipart/form-data" method="post">
								<div class="ase-file-wrapper">
		                        	<input name="ase-file" id="ase-file" type="file" />
		                        	<div class="ase-file-fake"></div>
								</div>
		                    </form>
		                    <form id="ase-save-form" action="index.php" enctype="multipart/form-data" method="post">
								<button id="ase-save"><span>Save Palette</span></button>
								<input type="hidden" value="" name="ase-colors-list" id="ase-colors-list" />
							</form>
						</div>
                        <iframe id="ase-load-target" name="ase-load-target"></iframe>
						<iframe id="ase-save-target" name="ase-save-target"></iframe>
                </div>
			</div>
			<div class="popup-tip">
				<div class="popup-tip-left"></div>
				<div class="popup-tip-mid">
					<span></span>
				</div>
				<div class="popup-tip-right"></div>
			</div>
		</div>
		<?php
		echo ob_get_clean();
	}
	
	function _jsInitialization($cookie_time) {
		global $gantry;
		
		$overlays = array();

        $__overlays = $gantry->retrieveTemp('overlays','overlays', array());
        $_templateCSSRules =  $gantry->retrieveTemp('overlays','templatecss', array());
        $_templateCSSVariations =  $gantry->retrieveTemp('overlays','templatecssvariations', array());
		
		foreach ($__overlays as $overlay => $list) {
			$overlays[$overlay] = "'none': {'file': 'overlay-off.png', 'value': 'none', 'name': 'Off', 'path': '".$gantry->templateUrl."/images/overlay-off.png'}, ";
			foreach ($list as $name => $file) {
				$overlays[$overlay] .= "'" . $file['name'] . "': {'file': '" . $file['file'] . "', 'value': '".$file['name']."', 'name': '".$name."', 'path': '".$gantry->templateUrl."/images/overlays/".$overlay."/".$file['file']."'}, ";
			}
			$overlays[$overlay] = substr($overlays[$overlay], 0, -2); 
		}
		
        if (substr($gantry->baseUrl, -1, 1) == '/') {
            $baseUrl = substr($gantry->baseUrl, 0, -1);
        }
        else {
            $baseUrl = $gantry->baseUrl;
        }



		$gantry->addInlineScript("
			
			var Crystalline = {
				cssRules: new Hash(".GantryJSON::encode($_templateCSSRules)."),
				varRules: new Hash(".GantryJSON::encode($_templateCSSVariations)."),
				savedCSS: new Hash({
					'main-background': '".$gantry->get('main-background')."',
					'header-background': '".$gantry->get('header-background')."',
					'header-text': '".$gantry->get('header-text')."',
					'header-link': '".$gantry->get('header-link')."',
					'feature-background': '".$gantry->get('feature-background')."',
					'feature-text': '".$gantry->get('feature-text')."',
					'feature-link': '".$gantry->get('feature-link')."',
					'body-background': '".$gantry->get('body-background')."',
					'body-text': '".$gantry->get('body-text')."',
					'body-link': '".$gantry->get('body-link')."',
					'bottom-background': '".$gantry->get('bottom-background')."',
					'bottom-text': '".$gantry->get('bottom-text')."',
					'bottom-link': '".$gantry->get('bottom-link')."',
					'header-overlay': '".$gantry->get('header-overlay')."',
					'main-text': '".$gantry->get('main-text')."',
					'main-link': '".$gantry->get('main-link')."',
					'main-overlay': '".$gantry->get('main-overlay')."',
					'feature-overlay': '".$gantry->get('feature-overlay')."',
					'body-overlay': '".$gantry->get('body-overlay')."',
					'bottom-overlay': '".$gantry->get('bottom-overlay')."',
					'header-graphic': '".$gantry->get('header-graphic')."',
					'header-shadows': '".$gantry->get('header-shadows')."',
					'main-shadows': '".$gantry->get('main-shadows')."',
					'feature-shadows': '".$gantry->get('feature-shadows')."',
					'body-shadows': '".$gantry->get('body-shadows')."',
					'bottom-shadows': '".$gantry->get('bottom-shadows')."'
				}),
				headers: new Hash({".$overlays['headers']."}),
				patterns: new Hash({".$overlays['patterns']."}),
				shadows: new Hash({
					'header-': ['rt-main-header'],
					'main-': ['rt-main-surround2'],
					'feature-': ['rt-utility', 'rt-feature'],
					'body-': ['rt-mainbody-bg'],
					'bottom-': ['rt-bottom', 'rt-main-footer']
				}),
				mooRainbow: {},
				sliders: {},
				templateName: '".$gantry->templateName."',
				baseUrl: '".$baseUrl."',				
				ajaxUrl: '".$gantry->getAjaxUrl()."',
				templateUrl: '".$gantry->templateUrl."',
				gantryURL: '".$gantry->gantryUrl."',
				aseLoadUrl: '".$gantry->baseUrl."?tmpl=ase-load',
				aseSaveUrl: '".$gantry->templateUrl."/ase-save.php',
				cookiePrefix: '".$gantry->template_prefix.$gantry->_base_params_checksum."',
				cookieTime: ".$cookie_time."
			}
		");
		
	}
}